package ir.classifiers;

import java.util.*;
import ir.vsr.*;

/**
 * Abstract class specifying the functionality of a classifier. Provides methods for 
 * training and testing a classifier
 *
 * @author       Sugato Basu
 */

public abstract class Classifier {
    
    /** To indicated whether this classifier uses inverted index */
    public abstract boolean usesInvertedIndex();

  /** For classifiers that use invertedIndices, this function sets 
   * them -- needed for efficient memory management */
    public abstract void setInvertedIndex(InvertedIndex index);
  
    /** To be overloaded by the extending class */
    public abstract String getName();

    /** Trains the classifier on the training examples */
    public abstract void train(Vector trainingExamples);
    
    /** Returns true if the predicted category of the test example matches the correct category,
	false otherwise */
    public abstract boolean test(Example testExample);

    /** Returns the array index with the maximum value 
     * @params Array whose index with max value has to be found
     */
    protected int argMax(double[] results){
	int maxIndex = 0;
	double max = results[0];
	
	for(int i=1;i<results.length;i++){
	    if(results[i]>max){
		max=results[i];
		maxIndex = i;
	    }
	}
	return(maxIndex);
    }
}
