/**
 * Title:        KNN
 * Description:  Simple K-Nearest Neighbour Classifier
 * Copyright:    Copyright (c) <p>
 * Company:    
 * @author       Sugato Basu
 * @version 1.0
 */

package ir.classifiers;

import java.io.*;
import java.util.*;
import ir.vsr.*;
import ir.utilities.*;

public class KNN extends Classifier
{
  protected InvertedIndex trainDocs = null;
  protected int K;
  public boolean debug = false;
  public final String name = "KNN";

  /** Function to indicate that this class uses an inverted index */
  public boolean usesInvertedIndex() {
    return true;
  }
  
  KNN(int k, boolean d) {
    K = k;
    debug = d;
  }

  public void setInvertedIndex(InvertedIndex index) {
    trainDocs = index;
  }

  /** Returns the name */
  public String getName() {
    return name;
  }
  
  public void train(Vector train) {
    if (trainDocs == null) {
      System.out.println("Please set inverted index using setInvertedIndex");
      System.exit(0);
    }
  }
  
  public boolean test(Example testExample) {
    HashMapVector testVector = testExample.getHashMapVector();
    Retrieval [] retrievals = trainDocs.retrieve(testVector);
    
    int k = Math.min(K,retrievals.length);
    double [] results = new double[CVLearningCurve.CLASSES.length];
    for (int i=0; i< results.length; i++) 
      results[i] = 0;
    for (int i=0; i<k; i++) {
      String name = retrievals[i].docRef.file.getName();
      results[CVLearningCurve.findClassID(name)]++;
    }
    if(debug) {
      System.out.print("Results: ");
      for (int i=0; i< results.length; i++) 
	System.out.print(CVLearningCurve.CLASSES[i] + "(" + results[i]+ ") ");
      System.out.println();
      System.out.println("Correct class: " + testExample.getCategory() + ", Predicted class: " + argMax(results));
    }
    for (int i=0; i<retrievals.length; i++)
      retrievals[i] = null;
    return (argMax(results) == testExample.getCategory());
  }
}
