/**
 * Title:        Rocchio
 * Description:  Simple Rocchio Classifier
 * @author       Sugato Basu
 */

package ir.classifiers;

import java.io.*;
import java.util.*;
import ir.vsr.*;
import ir.utilities.*;

public class Rocchio extends Classifier
{
  public String name = "Rocchio";
  protected InvertedIndex trainDocs = null;
  protected int K;
  protected ArrayList rocchioMeanVectors = null;
  boolean debug = false;
  boolean isNeg = false;

  /** Function to indicate that this class uses an inverted index */
  public boolean usesInvertedIndex() {
    return true;
  }

    
  Rocchio(int k, boolean neg, boolean d) {
    K = k;
    isNeg = neg;
    if (isNeg)
      name = "NegRocchio";
    debug = d;
  }

  public void setInvertedIndex(InvertedIndex index) {
    trainDocs = index;
  }
    
  /** Returns the name */
  public String getName() {
    return name;
  }
  
  public void train(Vector train) {
    ArrayList rocchioMeanFeedbacks = null;
    if (rocchioMeanVectors != null)
      rocchioMeanVectors.clear();
    if (debug) {
      System.out.println("\nTraining on:\n");
      System.out.println(train);
    }
    if(trainDocs == null) {
      System.out.println("Please set inverted index first using setInvertedIndex");
      System.exit(0);
    }
    for (int i=0; i<CVLearningCurve.CLASSES.length; i++) {
      HashMapVector dummy = (new TextStringDocument("",false)).hashMapVector();
      Feedback fdback = new Feedback(dummy,null,trainDocs);
      if(rocchioMeanFeedbacks == null)
	rocchioMeanFeedbacks = new ArrayList(CVLearningCurve.CLASSES.length);
      rocchioMeanFeedbacks.add(i,fdback);
    }
    
    for (int i=0; i<CVLearningCurve.CLASSES.length; i++) {
      if (debug)
	System.out.println("Training for class: " + CVLearningCurve.CLASSES[i] + "...");
      for (int j=0; j < trainDocs.docRefs.size(); j++) {
	Example trainExample = (Example) train.get(j);
	DocumentReference docRef = (DocumentReference) trainDocs.docRefs.get(j);
	if(trainExample.category == i) {
	  if (debug) 
	    System.out.println("Adding good file: " + docRef.file.getName());
	  ((Feedback) rocchioMeanFeedbacks.get(i)).addGood((DocumentReference) trainDocs.docRefs.get(j));
	}
	else if (isNeg){
	  if(debug)
	    System.out.println("Adding bad file: " + docRef.file.getName());
	  ((Feedback) rocchioMeanFeedbacks.get(i)).addBad((DocumentReference) trainDocs.docRefs.get(j));
	}
      }
      if (rocchioMeanVectors == null)
	rocchioMeanVectors = new ArrayList(CVLearningCurve.CLASSES.length);
      rocchioMeanVectors.add(i,((Feedback) rocchioMeanFeedbacks.get(i)).newQuery());
    }
  }
    
  public boolean test(Example testExample) {
    HashMapVector testVector = testExample.getHashMapVector();
    double [] results = new double[CVLearningCurve.CLASSES.length];

    for(int i=0; i<CVLearningCurve.CLASSES.length; i++) {
      results[i] = ((HashMapVector) rocchioMeanVectors.get(i)).cosineDistanceTo(testVector);
    }
    int predictedClass = argMax(results);
    if (debug) {
      System.out.print("Document: " + testExample.name + "\nResults: ");
      for (int j=0; j<CVLearningCurve.CLASSES.length; j++) {
	System.out.print(CVLearningCurve.CLASSES[j] + "(" + results[j] + ")\t");
      }
      System.out.println("\nCorrect class: " + testExample.getCategory() + ", Predicted class: " + predictedClass  + "\n");
    }
    return (predictedClass == testExample.getCategory());
  }
}


