package ir.classifiers;

import java.util.*;

/**
 * Wrapper class to test classifiers using k-fold CV.
 * Running it with optional -debug option gives very detailed output, 
 * useful for debugging
 * Required option: -method "classifier", where classifier can be:
 * 1) bayes 2) knn 3) rocchio 4) negrocchio
 *
 * @author       Sugato Basu
 */

public class TestClassifier {
  public static void main(String args[]) throws Exception
  {
    String dirName = "/u/mooney/ir-code/corpora/yahoo-science/";
    String [] classes = CVLearningCurve.CLASSES;
    boolean debug = false;
    String classifierName = null;
    Classifier classifier = null;
    // setting debug flag gives very detailed output, suitable for debugging
    for (int i=0; i<args.length; i++) {
      if (args[i].equals("-debug"))
	debug = true;
      else
	debug = false;
      if (args[i].equals("-method")) {
	classifierName = args[i+1];
      }
    }
    
    if (classifierName == null) {
      throw new Exception("\nPlease specify classifierName with -method option (choices: bayes, rocchio, negrocchio, knn)");
    }

    if (classifierName.equals("bayes"))
      classifier = new NaiveBayes(classes, debug);
    else if (classifierName.equals("rocchio"))
      classifier = new Rocchio(3,false,false);
    else if (classifierName.equals("negrocchio"))
      classifier = new Rocchio(3,true,false);
    else if (classifierName.equals("knn"))
      classifier = new KNN(5,false);

    if (classifier == null) {
      throw new Exception("\nPlease choose classifierName from the following choices: bayes, rocchio, negrocchio, knn");
    }
    
    System.out.println("\nInitializing classifier: " + classifier.getName());
    // Perform 10-fold cross validation to generate learning curve
    CVLearningCurve cvCurve = new CVLearningCurve(10,classifier,classes.length,dirName,debug);
    System.out.println("\nGenerating 10 fold CV learning curves for: " + classifier.getName());
    Vector result = cvCurve.getCVPredictions();
    System.out.println(result);
    System.out.println("\nTraining time in milliseconds: " + cvCurve.trainTime);
    System.out.println("\nTesting  time in milliseconds: " + cvCurve.testTime);
    // Create Gnuplot of learning curve
    cvCurve.makeGnuplotFile(result,classifier.getName());	
    System.out.println("\nGNUPLOT file is " + classifier.getName() + ".gplot");
  }
}
