package ir.vsr;

import java.util.*;

/** A lightweight object for storing information about positions of a token (a.k.a word, term)
 * in some document.
 *
 * @author Ray Mooney
 */

public class TokenPositionInfo implements java.lang.Comparable 
{
    /** The token itself */
    public String token;
    /** A list of positions (Integers) where the token occurs */
    public ArrayList positions;

    /** Create an initially empty data structure */
    public TokenPositionInfo(String token, ArrayList positions) {
	this.token = token;
	this.positions = positions;
    }

    /** Compares this TokenPositionInfo to another for sorting from tokens whose
     * first appearance in the document is earlier to those that first appear later.
     * @param obj The TokenPositionInfo to compare with.
     * @return -1 if smaller than obj, 0 if same, 1 if greater than obj
     * @see Arrays.sort
     */
    public int compareTo(Object obj) {
	TokenPositionInfo tokenPos = (TokenPositionInfo)obj;
	int thisFirstPos = ((Integer)positions.get(0)).intValue();
	int thatFirstPos = ((Integer)tokenPos.positions.get(0)).intValue();
	if (thisFirstPos == thatFirstPos)
	    return 0;
	else if (thisFirstPos < thatFirstPos)
	    return -1;
	else return 1;
    }

}
