package rmn;

import java.io.Serializable;

public class PotentialFactory implements Serializable {

  // weight matrix
  protected Matrix m_weights;
  
  // true counts matrix
  protected Matrix m_matTrue;
  // inference counts matrix
  protected Matrix m_matInf;

  // # of instantiated potentials
  int m_nPots;

  // true if further used in inference
  boolean m_bInference;

  public PotentialFactory()
  {
    m_weights = m_matTrue = m_matInf = null;
    m_nPots = 0;
    m_bInference = true;
  }

  public PotentialFactory(Matrix matrix)
  {
    m_weights = matrix;
    m_matTrue = m_matInf = null;
    m_nPots = 0;
    m_bInference = true;
  }

  public Potential newInstance()
  {
    m_nPots++;
    return new Potential(this);
  }

  public Matrix getWeightMatrix()
  {
    return m_weights;
  }

  public Matrix newWeightMatrix()
  {
    return m_weights.newMatrix();
  }

  public int size()
  {
    return m_weights.size();
  }

  public int instanceCount()
  {
    return m_nPots;
  }

  public void setInference(boolean bInference)
  {
    m_bInference = bInference;
  }

  public void setLearning(boolean bLearning)
  {
    if (bLearning) {
      // allocate counts
      m_matTrue = m_weights.newMatrix();
      m_matInf = m_weights.newMatrix();
      initCounts();
    }
    else {
      // deallocate counts
      m_matTrue = m_matInf = null;
    }
  }

  public void initWeights()
  {
    m_weights.fill(1);
  }

  public void initCounts()
  {
    m_matTrue.fill(0);
    m_matInf.fill(0);
  }

  public void incTrueCounts(int[] pos)
  {
    m_matTrue.inc(pos);
  }

  public void incInfCounts(int[] pos)
  {
    m_matInf.inc(pos);
  }

  public void updateWeights()
  {
    // perceptron delta rule
    m_weights.add_sub(m_matTrue, m_matInf);
  }

  // by default, all potentials have a fix arity
  public boolean hasVariableArity()
  {
    return false;
  }

}
