package rmn;

import java.util.*;

public class VotedPerceptron {

  static public int MAX_ITERATIONS = 50;

  int m_T = MAX_ITERATIONS;

  FactorGraphModel m_fgm;
  Vector m_vecPF;
  
  public VotedPerceptron(FactorGraphModel fgm, Vector vecPF)
  {
    m_fgm = fgm;
    m_vecPF = vecPF;
  }

  public void setIterations(int T)
  {
    m_T = T;
  }


  public boolean train()
  {
    // allocate learning structures
    setLearning(true);

    // set all weights to one
    initWeights();

    for (int t = 0; t < m_T; t++) {
      System.out.println("VotedPerceptron iteration t = " + t);

      for (int i = 0; i < m_fgm.size(); i++) {
	System.out.println("\tFactorGraph fg = " + i);
	// set all counts to zero
	initCounts();

	FactorGraph fg = m_fgm.getFactorGraph(i);
	// hide inferred value for hidden variables
	fg.setInfHidden();

	// true counts -- later move this outside cycles
	fg.computeTrueCounts();

	// inference counts
	fg.setMPE();
	fg.computeInfCounts();

	// perceptron delta rule
	updateWeights();
      }
    }

    // deallocate learning structures
    setLearning(false);

    return true;
  }


  public void setLearning(boolean bLearning)
  {
    for (int i = 0; i < m_vecPF.size(); i++) {
      PotentialFactory pf = (PotentialFactory) m_vecPF.get(i);
      pf.setLearning(bLearning);
    }
  }

  protected void initWeights()
  {
    for (int i = 0; i < m_vecPF.size(); i++) {
      PotentialFactory pf = (PotentialFactory) m_vecPF.get(i);
      pf.initWeights();
    }
  }

  protected void initCounts()
  {
    for (int i = 0; i < m_vecPF.size(); i++) {
      PotentialFactory pf = (PotentialFactory) m_vecPF.get(i);
      pf.initCounts();
    }
  }

  protected void updateWeights()
  {
    for (int i = 0; i < m_vecPF.size(); i++) {
      PotentialFactory pf = (PotentialFactory) m_vecPF.get(i);
      pf.updateWeights();
    }
  }

}
